@@ -22,7 +22,6 @@ import static org.testng.Assert.assertEquals;
 import static org.testng.Assert.assertFalse;
 import static org.testng.Assert.assertTrue;
 import static org.testng.Assert.fail;
-import com.beust.jcommander.Parameter;
 import com.google.common.collect.Sets;
 import java.io.ByteArrayOutputStream;
 import java.io.File;
@@ -32,14 +31,16 @@ import java.io.IOException;
 import java.io.OutputStreamWriter;
 import java.io.PrintStream;
 import java.io.PrintWriter;
-import java.lang.reflect.Constructor;
+import java.io.StringWriter;
 import java.lang.reflect.Field;
 import java.lang.reflect.InvocationTargetException;
 import java.lang.reflect.Method;
 import java.util.Arrays;
+import lombok.Cleanup;
+import org.apache.pulsar.PulsarBrokerStarter.BrokerStarter;
 import org.apache.pulsar.broker.ServiceConfiguration;
-import org.apache.pulsar.docs.tools.CmdGenerateDocs;
 import org.testng.annotations.Test;
+import picocli.CommandLine.Option;
 
 @Test(groups = "broker")
 public class PulsarBrokerStarterTest {
@@ -282,12 +283,14 @@ public class PulsarBrokerStarterTest {
      */
     @Test
     public void testMainWithNoArgument() throws Exception {
-        try {
-            PulsarBrokerStarter.main(new String[0]);
-            fail("No argument to main should've raised FileNotFoundException for no broker config!");
-        } catch (FileNotFoundException e) {
-            // code should reach here.
-        }
+        BrokerStarter brokerStarter = new BrokerStarter();
+        @Cleanup
+        StringWriter err = new StringWriter();
+        @Cleanup
+        PrintWriter printWriter = new PrintWriter(err);
+        brokerStarter.getCommander().setErr(printWriter);
+        assertEquals(brokerStarter.start(new String[0]), 1);
+        assertTrue(err.toString().contains("FileNotFoundException"));
     }
 
     /**
@@ -296,16 +299,16 @@ public class PulsarBrokerStarterTest {
      */
     @Test
     public void testMainRunBookieAndAutoRecoveryNoConfig() throws Exception {
-        try {
-            File testConfigFile = createValidBrokerConfigFile();
-            String[] args = {"-c", testConfigFile.getAbsolutePath(), "-rb", "-ra", "-bc", ""};
-            PulsarBrokerStarter.main(args);
-            fail("No Config file for bookie auto recovery should've raised IllegalArgumentException!");
-        } catch (IllegalArgumentException e) {
-            // code should reach here.
-            e.printStackTrace();
-            assertEquals(e.getMessage(), "No configuration file for Bookie");
-        }
+        File testConfigFile = createValidBrokerConfigFile();
+        String[] args = {"-c", testConfigFile.getAbsolutePath(), "-rb", "-ra", "-bc", ""};
+        BrokerStarter starter = new BrokerStarter();
+        @Cleanup
+        StringWriter err = new StringWriter();
+        @Cleanup
+        PrintWriter printWriter = new PrintWriter(err);
+        starter.getCommander().setErr(printWriter);
+        assertEquals(starter.start(args), 1);
+        assertTrue(err.toString().contains("No configuration file for Bookie"));
     }
 
     /**
@@ -314,15 +317,16 @@ public class PulsarBrokerStarterTest {
      */
     @Test
     public void testMainRunBookieRecoveryNoConfig() throws Exception {
-        try {
-            File testConfigFile = createValidBrokerConfigFile();
-            String[] args = {"-c", testConfigFile.getAbsolutePath(), "-ra", "-bc", ""};
-            PulsarBrokerStarter.main(args);
-            fail("No Config file for bookie auto recovery should've raised IllegalArgumentException!");
-        } catch (IllegalArgumentException e) {
-            // code should reach here.
-            assertEquals(e.getMessage(), "No configuration file for Bookie");
-        }
+        File testConfigFile = createValidBrokerConfigFile();
+        String[] args = {"-c", testConfigFile.getAbsolutePath(), "-ra", "-bc", ""};
+        BrokerStarter starter = new BrokerStarter();
+        @Cleanup
+        StringWriter err = new StringWriter();
+        @Cleanup
+        PrintWriter printWriter = new PrintWriter(err);
+        starter.getCommander().setErr(printWriter);
+        assertEquals(starter.start(args), 1);
+        assertTrue(err.toString().contains("No configuration file for Bookie"));
     }
 
     /**
@@ -330,15 +334,16 @@ public class PulsarBrokerStarterTest {
      */
     @Test
     public void testMainRunBookieNoConfig() throws Exception {
-        try {
-            File testConfigFile = createValidBrokerConfigFile();
-            String[] args = {"-c", testConfigFile.getAbsolutePath(), "-rb", "-bc", ""};
-            PulsarBrokerStarter.main(args);
-            fail("No Config file for bookie should've raised IllegalArgumentException!");
-        } catch (IllegalArgumentException e) {
-            // code should reach here
-            assertEquals(e.getMessage(), "No configuration file for Bookie");
-        }
+        File testConfigFile = createValidBrokerConfigFile();
+        String[] args = {"-c", testConfigFile.getAbsolutePath(), "-rb", "-bc", ""};
+        BrokerStarter starter = new BrokerStarter();
+        @Cleanup
+        StringWriter err = new StringWriter();
+        @Cleanup
+        PrintWriter printWriter = new PrintWriter(err);
+        starter.getCommander().setErr(printWriter);
+        assertEquals(starter.start(args), 1);
+        assertTrue(err.toString().contains("No configuration file for Bookie"));
     }
 
     /**
@@ -346,14 +351,16 @@ public class PulsarBrokerStarterTest {
      */
     @Test
     public void testMainEnableRunBookieThroughBrokerConfig() throws Exception {
-        try {
-            File testConfigFile = createValidBrokerConfigFile();
-            String[] args = {"-c", testConfigFile.getAbsolutePath()};
-            PulsarBrokerStarter.main(args);
-            fail("No argument to main should've raised IllegalArgumentException for no bookie config!");
-        } catch (IllegalArgumentException e) {
-            // code should reach here.
-        }
+        File testConfigFile = createValidBrokerConfigFile();
+        String[] args = {"-c", testConfigFile.getAbsolutePath()};
+        BrokerStarter starter = new BrokerStarter();
+        @Cleanup
+        StringWriter err = new StringWriter();
+        @Cleanup
+        PrintWriter printWriter = new PrintWriter(err);
+        starter.getCommander().setErr(printWriter);
+        assertEquals(starter.start(args), 1);
+        assertTrue(err.toString().contains("IllegalArgumentException"));
     }
 
     @Test
@@ -364,21 +371,15 @@ public class PulsarBrokerStarterTest {
             System.setOut(new PrintStream(baoStream));
 
             Class argumentsClass = Class.forName("org.apache.pulsar.PulsarBrokerStarter$StarterArguments");
-            Constructor constructor = argumentsClass.getDeclaredConstructor();
-            constructor.setAccessible(true);
-            Object obj = constructor.newInstance();
-
-            CmdGenerateDocs cmd = new CmdGenerateDocs("pulsar");
-            cmd.addCommand("broker", obj);
-            cmd.run(null);
+            PulsarBrokerStarter.main(new String[]{"-g"});
 
             String message = baoStream.toString();
 
             Field[] fields = argumentsClass.getDeclaredFields();
             for (Field field : fields) {
-                boolean fieldHasAnno = field.isAnnotationPresent(Parameter.class);
+                boolean fieldHasAnno = field.isAnnotationPresent(Option.class);
                 if (fieldHasAnno) {
-                    Parameter fieldAnno = field.getAnnotation(Parameter.class);
+                    Option fieldAnno = field.getAnnotation(Option.class);
                     String[] names = fieldAnno.names();
                     String nameStr = Arrays.asList(names).toString();
                     nameStr = nameStr.substring(1, nameStr.length() - 1);
