@@ -252,10 +252,25 @@ public class LeastResourceUsageWithWeightTest {
 
             }
 
+            @Override
+            public void start() throws LoadDataStoreException {
+
+            }
+
+            @Override
+            public void init() throws IOException {
+
+            }
+
             @Override
             public void startTableView() throws LoadDataStoreException {
 
             }
+
+            @Override
+            public void startProducer() throws LoadDataStoreException {
+
+            }
         };
 
         doReturn(conf).when(ctx).brokerConfiguration();
