@@ -22,16 +22,15 @@ import io.netty.channel.Channel;
 import io.netty.channel.ChannelHandler;
 import io.netty.channel.ChannelInitializer;
 import io.netty.channel.socket.SocketChannel;
+import io.netty.handler.codec.LengthFieldBasedFrameDecoder;
 import io.netty.handler.flush.FlushConsolidationHandler;
 import io.netty.handler.proxy.Socks5ProxyHandler;
 import io.netty.handler.ssl.SslContext;
 import io.netty.handler.ssl.SslHandler;
 import io.netty.handler.ssl.SslProvider;
 import java.net.InetSocketAddress;
-import java.util.Map;
 import java.util.Objects;
 import java.util.concurrent.CompletableFuture;
-import java.util.concurrent.ConcurrentHashMap;
 import java.util.concurrent.TimeUnit;
 import java.util.function.Supplier;
 import lombok.Getter;
@@ -43,7 +42,6 @@ import org.apache.pulsar.client.impl.conf.ClientConfigurationData;
 import org.apache.pulsar.client.util.ObjectCache;
 import org.apache.pulsar.common.protocol.ByteBufPair;
 import org.apache.pulsar.common.protocol.Commands;
-import org.apache.pulsar.common.protocol.FrameDecoderUtil;
 import org.apache.pulsar.common.util.SecurityUtility;
 import org.apache.pulsar.common.util.keystoretls.NettySSLContextAutoRefreshBuilder;
 import org.apache.pulsar.common.util.netty.NettyFutureUtil;
@@ -61,9 +59,9 @@ public class PulsarChannelInitializer extends ChannelInitializer<SocketChannel>
     private final InetSocketAddress socks5ProxyAddress;
     private final String socks5ProxyUsername;
     private final String socks5ProxyPassword;
-    private final ClientConfigurationData conf;
-    private Map<String, Supplier<SslContext>> sslContextSupplierMap;
-    private Map<String, NettySSLContextAutoRefreshBuilder> nettySSLContextAutoRefreshBuilderMap;
+
+    private final Supplier<SslContext> sslContextSupplier;
+    private NettySSLContextAutoRefreshBuilder nettySSLContextAutoRefreshBuilder;
 
     private static final long TLS_CERTIFICATE_CACHE_MILLIS = TimeUnit.MINUTES.toMillis(1);
 
@@ -78,32 +76,15 @@ public class PulsarChannelInitializer extends ChannelInitializer<SocketChannel>
         this.socks5ProxyPassword = conf.getSocks5ProxyPassword();
 
         this.tlsEnabledWithKeyStore = conf.isUseKeyStoreTls();
-        this.conf = conf.clone();
-        this.sslContextSupplierMap = new ConcurrentHashMap<>();
-        this.nettySSLContextAutoRefreshBuilderMap = new ConcurrentHashMap<>();
-    }
 
-    @Override
-    public void initChannel(SocketChannel ch) throws Exception {
-        ch.pipeline().addLast("consolidation", new FlushConsolidationHandler(1024, true));
-
-        // Setup channel except for the SsHandler for TLS enabled connections
-        ch.pipeline().addLast("ByteBufPairEncoder", ByteBufPair.getEncoder(tlsEnabled));
-        FrameDecoderUtil.addFrameDecoder(ch.pipeline(), Commands.DEFAULT_MAX_MESSAGE_SIZE);
-        ChannelHandler clientCnx = clientCnxSupplier.get();
-        ch.pipeline().addLast("handler", clientCnx);
-    }
-
-    private NettySSLContextAutoRefreshBuilder getNettySSLContextAutoRefreshBuilder(String host)
-            throws PulsarClientException {
-        if (tlsEnabledWithKeyStore) {
-            AuthenticationDataProvider authData1 = conf.getAuthentication().getAuthData(host);
-            if (StringUtils.isBlank(conf.getTlsTrustStorePath())) {
-                throw new PulsarClientException("Failed to create TLS context, the tlsTrustStorePath"
-                        + " need to be configured if useKeyStoreTls enabled");
-            }
-            return nettySSLContextAutoRefreshBuilderMap.computeIfAbsent(host,
-                    key -> new NettySSLContextAutoRefreshBuilder(
+        if (tlsEnabled) {
+            if (tlsEnabledWithKeyStore) {
+                AuthenticationDataProvider authData1 = conf.getAuthentication().getAuthData();
+                if (StringUtils.isBlank(conf.getTlsTrustStorePath())) {
+                    throw new PulsarClientException("Failed to create TLS context, the tlsTrustStorePath"
+                            + " need to be configured if useKeyStoreTls enabled");
+                }
+                nettySSLContextAutoRefreshBuilder = new NettySSLContextAutoRefreshBuilder(
                             conf.getSslProvider(),
                             conf.isTlsAllowInsecureConnection(),
                             conf.getTlsTrustStoreType(),
@@ -115,55 +96,67 @@ public class PulsarChannelInitializer extends ChannelInitializer<SocketChannel>
                             conf.getTlsCiphers(),
                             conf.getTlsProtocols(),
                             TLS_CERTIFICATE_CACHE_MILLIS,
-                            authData1));
-        }
-        throw new PulsarClientException(
-                "Failed to create TLS context, the tlsEnabledWithKeyStore need to be true");
-    }
-
-    private Supplier<SslContext> getSslContextSupplier(String host) {
-        return sslContextSupplierMap.computeIfAbsent(host, key -> new ObjectCache<>(() -> {
-            try {
-                SslProvider sslProvider = null;
-                if (conf.getSslProvider() != null) {
-                    sslProvider = SslProvider.valueOf(conf.getSslProvider());
-                }
-
-                // Set client certificate if available
-                AuthenticationDataProvider authData = conf.getAuthentication().getAuthData(host);
-                if (authData.hasDataForTls()) {
-                    return authData.getTlsTrustStoreStream() == null
-                            ? SecurityUtility.createNettySslContextForClient(
-                            sslProvider,
-                            conf.isTlsAllowInsecureConnection(),
-                            conf.getTlsTrustCertsFilePath(),
-                            authData.getTlsCertificates(),
-                            authData.getTlsPrivateKey(),
-                            conf.getTlsCiphers(),
-                            conf.getTlsProtocols())
-                            : SecurityUtility.createNettySslContextForClient(sslProvider,
-                            conf.isTlsAllowInsecureConnection(),
-                            authData.getTlsTrustStoreStream(),
-                            authData.getTlsCertificates(), authData.getTlsPrivateKey(),
-                            conf.getTlsCiphers(),
-                            conf.getTlsProtocols());
-                } else {
-                    return SecurityUtility.createNettySslContextForClient(
-                            sslProvider,
-                            conf.isTlsAllowInsecureConnection(),
-                            conf.getTlsTrustCertsFilePath(),
-                            conf.getTlsCertificateFilePath(),
-                            conf.getTlsKeyFilePath(),
-                            conf.getTlsCiphers(),
-                            conf.getTlsProtocols());
-                }
-            } catch (Exception e) {
-                throw new RuntimeException("Failed to create TLS context", e);
+                            authData1);
             }
-        }, TLS_CERTIFICATE_CACHE_MILLIS, TimeUnit.MILLISECONDS));
+
+            sslContextSupplier = new ObjectCache<SslContext>(() -> {
+                try {
+                    SslProvider sslProvider = null;
+                    if (conf.getSslProvider() != null) {
+                        sslProvider = SslProvider.valueOf(conf.getSslProvider());
+                    }
+
+                    // Set client certificate if available
+                    AuthenticationDataProvider authData = conf.getAuthentication().getAuthData();
+                    if (authData.hasDataForTls()) {
+                        return authData.getTlsTrustStoreStream() == null
+                                ? SecurityUtility.createNettySslContextForClient(
+                                sslProvider,
+                                conf.isTlsAllowInsecureConnection(),
+                                conf.getTlsTrustCertsFilePath(),
+                                authData.getTlsCertificates(),
+                                authData.getTlsPrivateKey(),
+                                conf.getTlsCiphers(),
+                                conf.getTlsProtocols())
+                                : SecurityUtility.createNettySslContextForClient(sslProvider,
+                                conf.isTlsAllowInsecureConnection(),
+                                authData.getTlsTrustStoreStream(),
+                                authData.getTlsCertificates(), authData.getTlsPrivateKey(),
+                                conf.getTlsCiphers(),
+                                conf.getTlsProtocols());
+                    } else {
+                        return SecurityUtility.createNettySslContextForClient(
+                                sslProvider,
+                                conf.isTlsAllowInsecureConnection(),
+                                conf.getTlsTrustCertsFilePath(),
+                                conf.getTlsCertificateFilePath(),
+                                conf.getTlsKeyFilePath(),
+                                conf.getTlsCiphers(),
+                                conf.getTlsProtocols());
+                    }
+                } catch (Exception e) {
+                    throw new RuntimeException("Failed to create TLS context", e);
+                }
+            }, TLS_CERTIFICATE_CACHE_MILLIS, TimeUnit.MILLISECONDS);
+        } else {
+            sslContextSupplier = null;
+        }
     }
 
-    /**
+    @Override
+    public void initChannel(SocketChannel ch) throws Exception {
+        ch.pipeline().addLast("consolidation", new FlushConsolidationHandler(1024, true));
+
+        // Setup channel except for the SsHandler for TLS enabled connections
+        ch.pipeline().addLast("ByteBufPairEncoder", ByteBufPair.getEncoder(tlsEnabled));
+
+        ch.pipeline().addLast("frameDecoder", new LengthFieldBasedFrameDecoder(
+                Commands.DEFAULT_MAX_MESSAGE_SIZE + Commands.MESSAGE_SIZE_FRAME_PADDING, 0, 4, 0, 4));
+        ChannelHandler clientCnx = clientCnxSupplier.get();
+        ch.pipeline().addLast("handler", clientCnx);
+    }
+
+   /**
      * Initialize TLS for a channel. Should be invoked before the channel is connected to the remote address.
      *
      * @param ch      the channel
@@ -182,10 +175,9 @@ public class PulsarChannelInitializer extends ChannelInitializer<SocketChannel>
         ch.eventLoop().execute(() -> {
             try {
                 SslHandler handler = tlsEnabledWithKeyStore
-                        ? new SslHandler(getNettySSLContextAutoRefreshBuilder(sniHost.getHostName()).get()
-                        .createSSLEngine(sniHost.getHostString(), sniHost.getPort()))
-                        : getSslContextSupplier(sniHost.getHostName()).get()
-                        .newHandler(ch.alloc(), sniHost.getHostString(), sniHost.getPort());
+                        ? new SslHandler(nettySSLContextAutoRefreshBuilder.get()
+                                .createSSLEngine(sniHost.getHostString(), sniHost.getPort()))
+                        : sslContextSupplier.get().newHandler(ch.alloc(), sniHost.getHostString(), sniHost.getPort());
 
                 if (tlsHostnameVerificationEnabled) {
                     SecurityUtility.configureSSLHandler(handler);
